/*
 * Modification History
 *
 * 2002-February-25    Jason Rohrer
 * Created.  
 */



#include "Log.h"



const int Log::DEACTIVATE_LEVEL = 0;

const int Log::CRITICAL_ERROR_LEVEL = 1;

const int Log::ERROR_LEVEL = 2;

const int Log::WARNING_LEVEL = 3;

const int Log::INFO_LEVEL = 4;

const int Log::DETAIL_LEVEL = 5;

const int Log::TRACE_LEVEL = 6;



Log::~Log() {

    }
