/*
 * Modification History
 *
 * 2001-September-15		Jason Rohrer
 * Created.
 */
 
 
#ifndef GUI_COMPONENT_INCLUDED
#define GUI_COMPONENT_INCLUDED 


#include "minorGems/ui/GUIComponent.h"


/**
 * An generic superclass for all gui components.
 * Allows for easy event source determination during event passing.
 *
 * @author Jason Rohrer
 */
class GUIComponent {
		
	};



#endif



