// Jason Rohrer
// keyCodes.h

/**
*
*	Header for defining key codes for various platforms
*
*
*	Created 12-15-99
*	Mods:	
*		Jason Rohrer	3-23-2000	Added more key codes
*
*/


#ifdef WINDOWS_KEY_CODES

	#define M_KEY 	0x4D
	#define N_KEY	0x4E
	
	#define S_KEY	0x53
	
	#define Q_KEY	0x51
	
	#define L_KEY	0x4C
	
	#define R_KEY	0x52
	
	#define T_KEY	0x54

#endif




#ifdef MAC_KEY_CODES

	#define M_KEY 	0x2E
	#define N_KEY	0x2D
	
	#define S_KEY	0x01
	
	#define Q_KEY	0x0C
	
	
	#define L_KEY	0x25
	
	#define R_KEY	0x0F
	
	#define T_KEY	0x11

#endif